#include <stdlib.h>
#include <string.h>
#include "libs.h"
#include "log.h"
#include "db.h"

static int try_icl_(int logdepth, const char *prefix, const char *test_c_in, char *includes, char *cflags, char *ldflags, int run)
{
	char *out = NULL;
	char *tmp, *inc;
	const char *test_c;
	char c[1024];
	int l, compres;

	if (includes != NULL) {
		l = strlen(includes);
		memcpy(c, includes, l);
		c[l] = '\n';
		l++;
		strcpy(c+l, test_c_in);
		test_c = c;
	}
	else
		test_c = test_c_in;

	logprintf(logdepth, "trying '%s' and '%s' and '%s', %s\n", includes == NULL ? "" : includes, cflags, ldflags, run ? "with a run" : "with no run");

	if (run)
		compres = compile_run(logdepth+1, test_c, NULL, cflags, ldflags, &out);
	else {
		char *fn_output = NULL;
		compres = compile_code(logdepth+1, test_c, &fn_output, NULL, cflags, ldflags);
		if (fn_output != NULL) {
			unlink(fn_output);
			free(fn_output);
		}
	}


	if (compres == 0) {
		if (!run || target_emu_fail(out) || (strncmp(out, "OK", 2) == 0)) {
			free(out);

			/* no prefix: don't modify the database, the caller will do that */
			if (prefix == NULL)
				return 1;

			if ((cflags != NULL) && (*cflags == '+'))
				cflags++;
			if ((ldflags != NULL) && (*ldflags == '+'))
				ldflags++;

			tmp = malloc(strlen(prefix) + 32);

			if (includes != NULL) {
				if (*includes == '\0')
					inc = strclone("");
				else
					inc = uniq_inc_str(includes, NULL, "\\n");
				sprintf(tmp, "%s/includes", prefix);
				put(tmp, inc);
			}
			else
				inc = NULL;

			if (cflags != NULL) {
				sprintf(tmp, "%s/cflags", prefix);
				if (*cflags == '+')
					put(tmp, cflags+1);
				else
					put(tmp, cflags);
			}

			if (ldflags != NULL) {
				sprintf(tmp, "%s/ldflags", prefix);
				if (*ldflags == '+')
					put(tmp, ldflags+1);
				else
					put(tmp, ldflags);
			}

			if (inc != NULL) {
				report("OK ('%s', '%s' and '%s')\n", inc, cflags, ldflags);
				free(inc);
			}
			else
				report("OK ('%s' and '%s')\n", cflags, ldflags);

			sprintf(tmp, "%s/presents", prefix);
			put(tmp, strue);
			free(tmp);
			return 1;
		}
		free(out);
	}
	return 0;
}

int try_icl(int logdepth, const char *prefix, const char *test_c_in, char *includes, char *cflags, char *ldflags)
{
	return try_icl_(logdepth, prefix, test_c_in, includes, cflags, ldflags, 1);
}

int try_icl_norun(int logdepth, const char *prefix, const char *test_c_in, char *includes, char *cflags, char *ldflags)
{
	return try_icl_(logdepth, prefix, test_c_in, includes, cflags, ldflags, 0);
}


int try_fail(int logdepth, const char *prefix)
{
	char *tmp;
	tmp = malloc(strlen(prefix) + 32);
	sprintf(tmp, "%s/presents", prefix);
	put(tmp, sfalse);
	free(tmp);
	report("not found\n");
	logprintf(logdepth, "NOT FOUND.");
	return 1;
}
