/*
    libporty - /etc/resolv.conf parser
    Copyright (C) 2012  Tibor 'Igor2' Palinkas

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Project page: http://repo.hu/projects/libporty
    Author: libporty@igor2.repo.hu
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <libporty/net/uninit_chain.h>
#include <libporty/net/dns4.h>
#include "net_mtime.h"
#include "net_resolvconf.h"

static char *resolvconf_fn = "/etc/resolv.conf";
static P_time_t resolvconf_seen = 0;

P_nameserver_t *P_net_nameservers = NULL;
static P_nameserver_t *last = NULL;

static char *strclone(const char *i)
{
	char *o;
	int l;

	l = strlen(i);
	o = malloc(l+1);
	memcpy(o, i, l+1);
	return o;
}

static void put(const char *ip)
{
	P_nameserver_t *n;

	n = malloc(sizeof(P_nameserver_t));
	n->ip = strclone(ip);
	n->next = NULL;

	if (P_net_nameservers == NULL)
		P_net_nameservers = n;
	else
		last->next = n;
	last = n;
}

/* whether uninit chain is already set up */
static in_chain = 0;

static void free_all(void)
{
	P_nameserver_t *n, *next;

	/* free all existing entries */
	for(n = P_net_nameservers; n != NULL; n = next) {
		next = n->next;
		free(n->ip);
		free(n);
	}
	P_net_nameservers = NULL;
	last = NULL;
}

static void uninit(void)
{
	free_all();
	in_chain = 0;
}

int P_net_resolv_conf_reread(void)
{
	FILE *f;
	char line[1024], *s, *end;

	if (!in_chain) {
		P_net_uninit_reg(uninit);
		in_chain = 1;
	}

	f = fopen(resolvconf_fn, "r");
	if (f == NULL)
		return -1;

	free_all();

	/* parse the file */
	while(!(feof(f))) {
		*line = '\0';
		fgets(line, sizeof(line), f);
		s = line;
		while((*s == ' ') || (*s == '\t')) s++;
		switch(*s) {
			case '#':
			case '\0':
			case '\r':
				break;
			default:
				if (strncmp(s, "nameserver", 10) != 0)
					break;
				s+=10;
				while((*s == ' ') || (*s == '\t')) s++;
				end = s+strlen(s)-1;
				while((end > s) && ((*end < '0') || (*end > '9'))) {
					*end = '\0';
					end--;
				}
				if (P_dns4_is_addr(s))
					put(s);
		}
	}
	fclose(f);
	return 0;
}

/* look at mtime fo the hosts file and reread file if it's different
   from what we've seen last */
void P_net_resolvconf_check_reread(void)
{
	P_time_t mt;
	mt = P_net_mtime(resolvconf_fn);
	if (mt != resolvconf_seen) {
		resolvconf_seen = mt;
		P_net_resolv_conf_reread();
	}
}

