# this plugin script lists all globals in 3 columns:
#  class (variable, function, etc)
#  ID
#  type info


globals_list() {
$TREECC "$@" | awk -v "only_target=$only_target" -v "target=$fn" '

# close a global section

BEGIN {
	FUN[0]  = "global-public-var"
	FUN[1]  = "global-public-fun"
	FUN[10] = "global-static-var"
	FUN[11] = "global-static-fun"

	FUN[100]  = "local-stack-var  "
	FUN[101]  = "local-stack-fun  "
	FUN[110]  = "local-static-var "
	FUN[111]  = "local-static-fun "
}

function cl() {
	lcl()
	global = 0;
	type = 0;
	fun = 0;
	static = 0
	vtype=""
	id = ""
	infunc=0
}

function lcl() {
	local = 0;
	ltype = 0;
	lfun = 0;
	lstatic = 0
	lvtype=""
	lid = ""
}

function lfinish_(lid)
{
	if ((!only_target) || (lid ~ target ":")) {
		print FUN[lfun+lstatic*10+100] " " lid " " lvtype
	}
}

function lfinish(    v,I,n)
{
	if (lid != "") {
		v = split(lid, I, ",")
		for(n = 1; n <= v; n++)
			lfinish_(I[n])
	}
	lcl()
}

function finish_(id)
{
	if ((!only_target) || (id ~ target ":"))
		print FUN[fun+static*10] " " id " " vtype
}

function finish(   v,I,n)
{
	lfinish()
	if (id != "") {
		v = split(id, I, ",")
		for(n = 1; n <= v; n++)
			finish_(I[n])
	}
	cl()
}

{
	sub("[[][0-9]+:[0-9]+\\]", "", $0)

	level = $0
	sub("[^ ].*$", "", level)
	level = length(level)
}

/^  [^ ]/ {
	# end of a global section
	finish()
}

### === global
# global declaration (variable or anything else)
/^  DECL/ { global = 1; }
/^  FUNCDEF/ { global = 1; infunc=1; }

# save id, reset type to 0
/^   ID/ {
	if (global) {
		id_ = $0
		sub("^ *ID *", "", id_)
		if (id == "")
			id = id_
		else
			id = id "," id_
		type = 0
		idlev = level
	}
	next
}

# type -1 is types before the id, type +1 is types after the id
/^    ?TYPE/ {
	if (global) {
		if (id == "")
			type = -1
		else
			type = 1
	}
	next
}

/^    CLASS static/ {
	if (type == -1)
		static=1
	next
}

# ARGLIST at this level after the id means function
/^     ARGLIST/ {
	if (type == 1)
		fun=1
	next
}

(type == -1) {
	vtype = vtype "|" $0
	next
}



### === local
# global declaration (variable or anything else)
/^  +DECL/ {
	if (infunc) {
		lfinish();
		local = 1;
		type=-2
	}
}

# save local id, reset type to 0
/^   +ID/ {
	if ((local) && (ltype != 0)) {
		lid_ = $0
		sub("^ *ID *", "", lid_)
		if (lid == "")
			lid = lid_
		else
			lid = lid "," lid_
		idlev = level;
	}
}

{
	if (level < idlev) {
		idlev = -1;
		ltype = 0;
	}
}

# type -1 is types before the id, type +1 is types after the id
/^    +TYPE/ {
	if (local) {
		if (lid == "")
			ltype = -1
		else
			ltype = 1
	}
}

/^    +CLASS static/ {
	if (ltype == -1)
		lstatic=1
}

# ARGLIST at this level after the id means function
/^     +ARGLIST/ {
	if (ltype == 1)
		lfun=1
	next
}

(ltype == -1) {
	lvtype = lvtype "|" $0
}

END {
	finish()
}
'
}

cmd_globals()
{
	local only_target mode

	only_target=1
	mode=normal
	while test ! -z "$1"
	do
		case "$1" in
			--all|-a) only_target=0 ;;
			--state|-s) mode=state;;
			--public|-p) mode=public;;
			*.c|*.h) fn="$1" ;;
			*) opts="$opts $1" ;;
		esac
		shift 1
	done

	case $mode in
		state) globals_list $opts $fn | grep "^global-.*-var\|local-static-var" ;;
		public) globals_list $opts $fn | grep "^global-public-" ;;
	 *) globals_list $opts $fn ;;
	esac
}

