#ifndef GENTREX_H
#define GENTREX_H 
typedef struct gentrex_state_s gentrex_state_t;

/* Node complementer: straight is the logical value (inverted is the opposite
   value) */
typedef struct gtx_nodecomp_s {
	void *straight;
	void *inverted;
} gentrex_nodecomp_t;

typedef enum gentrex_op_e {
	GTX_OP_EQ,      /* "==" (string equality) */
	GTX_OP_MATCH,   /* "~"  (pattern match) */
	GTX_OP_C_OP     /* custom operators (e.g. >=) */
} gentrex_op_t;

typedef struct gentrex_api_s {
	void *user_tree_ctx;   /* pointer to user data, used in tree callbacks; e.g. pointer to the root of the tree */
	void *user_map_ctx;    /* pointer to user data, used in map callbacks; e.g. pointer to a hash structure */

	/* map callbacks: store/retrieve node complementers by name; map_get() should not modify result if name is not found */
	void (*map_get)(void *user_map_ctx, const char *name, gentrex_nodecomp_t *result);
	void (*map_set)(void *user_map_ctx, const char *name, gentrex_nodecomp_t ndc);

	/* tree callbacks: return a relative node of the given node */
	void *(*child)(void *user_tree_ctx, void *node);
	void *(*parent)(void *user_tree_ctx, void *node);
	void *(*next)(void *user_tree_ctx, void *node);

	/* node tag matching */
	void *(*op_cookie_alloc)(void *user_tree_ctx, gentrex_op_t match, const char *tagname, const char *str_or_pattern, const char *custom_operator);
	void (*op_cookie_free)(void *user_tree_ctx, gentrex_op_t match, const char *tagname, const char *str_or_pattern, void *op_cookie, const char *custom_operator);
	int (*tag_eq)(void *user_tree_ctx, void *node, void *op_cookie, const char *tagname, const char *str);
	int (*tag_match)(void *user_tree_ctx, void *node, void *op_cookie, const char *tagname, const char *pattern);
	int (*tag_c_op)(void *user_tree_ctx, void *node, void *op_cookie, const char *tagname, const char *str, const char *c_op);

} gentrex_api_t;
#endif

/* Diagnostics: dump the expression bytecode of a state. */
void gentrex_dump(gentrex_state_t *ctx, void *tree_root);

/* Allocate a new state bound to an API, compile an expression script into
   bytecode. The same api can be used for multiple states. Returns the newly
   allocated state or NULL on error. */
gentrex_state_t *gentrex_alloc(const gentrex_api_t *api, const char *script, const char **error_msg);

/* Execute the first/next iteration of the state on a given subtree pointed by
   tree_root. Returns the result of the expression for the next iteration
   (might match a previous return value) or NULL if there's no more iterations
   to be done. Tree_root or the tree itself should not change in subseqent 
   calls (except if gentrex_reset is also called on the state) */
void *gentrex_execute(gentrex_state_t *ctx, void *tree_root);

/* Reset search state so the next iteration will be the first iteration again. */
void gentrex_reset(gentrex_state_t *ctx);

/* Free a parser state, reclaim all memory allocated */
void gentrex_free(gentrex_state_t *ctx);
